function results = sur_lag0(y,X,W)

% OBJETIVO: Estimacin del modelo SUR lag.
%--------------------------------------------------------------------------
% where: 
%        y   = an 'eq' structure containing dependent variables 
%              e.g. y(1).eq = y1; y(2).eq = y2; y(3).eq = y3;
%        x   = an 'eq' structure containing explanatory variables
%                 e.g. x(1).eq = [iota x1 x4]; 
%                      x(2).eq = [iota x1]; 
%                      x(3).eq = [iota x1 x2 x5];
% -------------------------------------------------------------------------
% Ref: Mur J, Lpez FA and M Herrera (2010) Testing for spatial effect in 
% Seemingly Unrelated Regressions. Spatial Economic Analysis 5(4) 399-440
% -------------------------------------------------------------------------
% Escrita en fecha 18/03/08 por:
% Fernando A. Lpez Hernndez
% Facultad de C.C. de la Empresa
% Dpto. Metodos Cuantitativos e Informticos
% Universidad Politcnica de Cartagena
% E-mail: fernando.lopez@upct.es
%--------------------------------------------------------------------------

[n k]=size(X(1).eq); % En principio, paneles equilibrados: mismo k para todo t
T=length(y);
info.itmax = 10;
info.crit=0.0001;
Rsur=sur(T,y,X,1);% toolbox de Lesage para obtener la estimacin max ver del modelo SUR.
% NOTA: la toolbox de LeSage tiene un error y nombra results(eq).yhat a los residuos y
% results(eq).resid a los valores previstos.

for i=1:T
    u(i).eq=olse(y(i).eq,X(i).eq);
    v(i).eq=ols(y(i).eq,X(i).eq);
    s(i).eq=u(i).eq'*u(i).eq/n;
    L(i)=-(n/2)*log(2*pi)-(n/2)*log(s(i).eq)-(1/(2*s(i).eq))*u(i).eq'*u(i).eq; % Obtencin de los valores de la FV para cada ecuacin
end
% Valores Lik para cada una de las ecuaciones temporales.
L;
% Apilamos los vectores 
XX=X(1).eq;
yy=y(1).eq;
B=Rsur(1).beta;
vv=v(1).eq.resid;
uu=u(1).eq;
re=Rsur(1).yhat;
reh=Rsur(1).yhat;
for i=2:T
    yy=vertcat(yy,y(i).eq);
    B=vertcat(B,Rsur(i).beta);
    XX=blkdiag(XX,X(i).eq);
    uu=horzcat(uu,u(i).eq);
    re=vertcat(re,Rsur(i).yhat);
    reh=horzcat(reh,Rsur(i).yhat);
    vv=horzcat(vv,v(i).eq.resid);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test de diagonalidad de la matriz de varianzas y cova
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Test LR de cociente de verosimiltudes para ver si los residuos estn correlacionados
S=Rsur(1).sigma;
SI=inv(S);
K = inv(kron(S,eye(n,n)));
r1=yy-XX*B;
LL=-(T*n/2)*log(2*pi)-(n/2)*log(det(S))-(1/2)*r1'*K*r1;
LR=-2*(sum(L)-LL);

% Test LM de esfericidad
r=corrcoef(vv);
LME=0;
for i=1:T-1
    for j=i+1:T
        LME=LME+n*r(i,j)^2;
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test de dependencia espacial
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM-SUR-Err
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

iota=ones(T,1);
U=Rsur(1).yhat;
for i=2:T
    U=horzcat(U,Rsur(i).yhat);
end
% Con la idea de Anselin 1988b
gerr=iota'*(SI.*(U'*W*U));
TTT=trace(W^2)*eye(T,T);
TTS=trace(W'*W)*ones(T,T);
LMSURerr=gerr*inv(TTT+TTS.*(SI.*S))*gerr';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM-SUR-Lag
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:T
    for j=1:T
        E(i,j).eq=zeros(T,T);
        E(i,j).eq(i,j)=1;
        E(i,j).eq(j,i)=1;
    end
end
% gradiente
YL=W*y(1).eq;
for i=2:T
    YL=horzcat(YL,W*y(i).eq);
end
glag=iota'*(SI.*(U'*YL));

% Matriz de Informacin
JI11=XX'*kron(SI,eye(n))*XX;
JI12=XX'*kron(SI*E(1,1).eq,W)*XX*B;
for i=2:T
    JI12=horzcat(JI12,XX'*kron(SI*E(i,i).eq,W)*XX*B);
end

for i=1:T
    for j=1:T
        if i==j 
            JI22(i,j)=SI(i,j)*(B'*XX'*kron(E(i,j).eq,W'*W)*XX*B+S(i,j)*trace(W'*W))+trace(W^2); 
        else
            JI22(i,j)=B'*XX'*kron(E(j,j).eq*SI*E(i,i).eq,W'*W)*XX*B + SI(i,j)*S(i,j)*trace(W'*W);
        end
    end
end
% Indexar los elementos diferentes de Sigma
c1=0;c2=0;c3=0;
for k1=1:T
    c2=c2+1;
    for i=1:T-k1+1
        c1=c1+1;c3=c3+1;
        f(c1)=c2;
        c(c1)=c3;
    end
    c3=c2;
end

for i=1:T
    for j=1:T*(T+1)/2
        JI23(i,j)=0;
    end
end

 for i=1:T*(T+1)/2
       for j=1:T*(T+1)/2   
       JI33(i,j)=(n/2)*trace(SI*E(f(i),c(i)).eq*SI*E(f(j),c(j)).eq);
    end
 end

LMSURlag=glag*inv(JI22-JI12'*inv(JI11)*JI12)*glag';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM-SUR-SARMA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SI11=XX'*kron(SI,eye(n))*XX;
SI12=XX'*kron(SI*E(1,1).eq,W)*XX*B;
for i=2:T
    SI12=horzcat(SI12,XX'*kron(SI*E(i,i).eq,W)*XX*B);
end
for i=1:T
    for j=1:T
        if i==j 
            SI22(i,j)=SI(i,j)*(B'*XX'*kron(E(i,j).eq,W'*W)*XX*B+S(i,j)*trace(W'*W))+trace(W^2); 
        else
            SI22(i,j)=B'*XX'*kron(E(j,j).eq*SI*E(i,i).eq,W'*W)*XX*B + SI(i,j)*S(i,j)*trace(W'*W);
        end
    end
end
SI23=TTT+TTS.*(SI.*S);
SI33=eye(T,T)*2*trace(W'*W);
gsarma=[glag gerr];
LMSURsarma=gsarma*inv([SI22-SI12'*inv(SI11)*SI12  SI23;SI23' SI33])*gsarma';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM*-SUR-Lag
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SI11=XX'*kron(SI,eye(n))*XX;
for i=1:T*(T+1)/2
   for j=1:T*(T+1)/2   
       SI44(i,j)=(n/2)*trace(SI*E(f(i),c(i)).eq*SI*E(f(j),c(j)).eq);
   end
end
Iff=[SI11 zeros(k*T,T*(T+1)/2);zeros(T*(T+1)/2,k*T) SI44];
Ilf=[SI12' zeros(T,T*(T+1)/2)];
Ilpf=SI22-Ilf*inv(Iff)*Ilf';


Irr=TTT+TTS.*(SI.*S);
Irl=eye(T,T)*2*trace(W'*W);
LMRSURlag=(glag'-Irl*inv(Irr)*gerr')'*inv(Ilpf-Irl*inv(Irr)*Irl)*(glag'-Irl*inv(Irr)*gerr');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM*-SUR-Err
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
LMRSURerr=(gerr'-Irl*inv(Ilpf)*glag')'*inv(Irr-Irl*inv(Ilpf)*Irl)*(gerr'-Irl*inv(Ilpf)*glag');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimacin del modelo SUR-lag
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Utilizamos el metodo de Ord(1975) para evitar problemas con autovaloeres complejos.
% Si W es simtrica sus autovalores son reales. En el caso de W
% estandarizada no podemos asegurar que sus autovalores sean reales.??????
av=eig(W);
for i=1:n
    if abs(av(i))<1e-15; avv(i)=0;
    else
        avv(i)=av(i);
    end
end

% Definicion opciones para el proceso optimizacion:
options = optimset('fminsearch');
% El valor de MaxFunEvals debe ser alto si el problema tiene muchos cortes
% temporales
% options = optimset('Display','iter')%
options = optimset('TolFun',1e-6,'MaxFunEvals',8000);
ini=zeros(1,T); 

% Para agilizar el proceso de optimizacin con la funcin fminsearch se
% toma como punto inicial los valores de rho para cada uno de los cortes
% temporales que se obtienen con las toolbox de LeSage
for i=1:T
    in=sar(y(i).eq,X(i).eq,W); %Toolbox de LeSage
    ini(i)=in.rho;
end
S0=Rsur(1).sigma;
%fprintf(1,'Punto inicial        = %9.4f \n',ini);
fprintf(1,'Punto inicial        =');fprintf(1,'%9.4f', ini);fprintf(1,'\n')
% Proceso iterativo para la obtencin de los estimadores:

maxit=20;  % nmero max de iteraciones
tol=1e-7; % precision de la estimacin en la iteraciones

% Proceso de estimacin iterativo

% Obtencion del minimo bajo la hip alternativa.
[mm,fvalh1]=fminsearch('f_sur_lag3',ini,options,T,yy,XX,W,S0,avv);
% fprintf(1,'iteracion          = %9.4f %9.4f   \n',mm);
fprintf(1,'iteracion            =');fprintf(1,'%9.4f', mm);fprintf(1,'\n')
% Proceso de estimacin iterativo
for i=1:maxit
G=diag(mm);
A=eye(n*T)-kron(G,W);
OMI=kron(inv(S0),eye(n));
B=inv(XX'*OMI*XX)*XX'*OMI*A*yy;
R=A*yy-XX*B;
RR=reshape(R,n,T);
S1=cov(RR);
[mm,fvalh1]=fminsearch('f_sur_lag3',ini,options,T,yy,XX,W,S1,avv);
fprintf(1,'iteracion            =');fprintf(1,'%9.4f', mm);fprintf(1,'\n')
%fprintf(1,'iteracion          = %9.4f %9.4f   \n',mm);
if sum((mm-ini).^4)<tol, break, else ini=mm;end
S0=S1;
end


G=diag(mm);
A=eye(n*T)-kron(G,W);
OMI=kron(inv(S1),eye(n));
B=inv(XX'*OMI*XX)*XX'*OMI*A*yy;
R=A*yy-XX*B;
RR=reshape(R,n,T);
SH1=cov(RR);
CH1=corrcoef(RR);
jj=[B' mm]';
results.jj=jj;
-fvalh1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Matriz de Informacin bajo H1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:T
    for j=1:T
        E(i,j).eq=zeros(T,T);
        E(i,j).eq(i,j)=1;
        E(i,j).eq(j,i)=1;
    end
end
SH1I=inv(SH1);

% Matriz de Informacin
H1I11=XX'*kron(SH1I,eye(n))*XX;
H1I12=XX'*kron(SH1I*E(1,1).eq,W)*inv(A)*XX*B;
for i=2:T
    H1I12=horzcat(H1I12,XX'*kron(SH1I*E(i,i).eq,W)*inv(A)*XX*B);
end
H1I13=zeros(k*T,T*(T+1)/2);

for i=1:T
    At(i).eq=eye(n)-mm(i)*W;
    for j=1:T
        H(i,j).eq=inv(A')*kron(E(j,j).eq*SH1I*E(i,i).eq,W'*W)*inv(A);
    end
end

for i=1:T
    for j=1:T
        if i==j 
            H1I22(i,j)=trace(inv(At(i).eq)*W*inv(At(i).eq)*W)+ B'*XX'*H(i,i).eq*XX*B+trace(H(i,i).eq*kron(SH1,eye(n))); 
        else
            H1I22(i,j)=B'*XX'*H(i,j).eq*XX*B + trace(H(i,j).eq*kron(SH1,eye(n)));
        end
    end
end
% Indexar los elementos diferentes de Sigma
c1=0;c2=0;c3=0;
for k1=1:T
    c2=c2+1;
    for i=1:T-k1+1
        c1=c1+1;c3=c3+1;
        f(c1)=c2;
        c(c1)=c3;
    end
    c3=c2;
end

for i=1:T
    for j=1:T*(T+1)/2
        H1I23(i,j)=trace(kron(E(i,i).eq,W)*inv(A)*kron(E(f(j),c(j)).eq*SH1I,eye(n)));
    end
end

 for i=1:T*(T+1)/2
       for j=1:T*(T+1)/2   
       H1I33(i,j)=(n/2)*trace(SH1I*E(f(i),c(i)).eq*SH1I*E(f(j),c(j)).eq);
    end
 end

IH1=[H1I11 H1I12 H1I13;H1I12' H1I22 H1I23;H1I13' H1I23' H1I33];
IIH1=inv(IH1);
tmp=sqrt(diag(IIH1));
tstat=jj./tmp(1:k*T+T);
pv=2*(1-tcdf(abs(tstat),n-k-1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     IMPRESION RESULTADOS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf(1,' \n');
fprintf(1,'Test of diagonality of error covariance matrix \n');
fprintf(1,' \n');
fprintf(1,'LM - Lagrange Multiplier = %9.4f   \n',LME);
fprintf(1,'LR - Likelihood Ratio = %9.4f   \n',LR);
fprintf(1,' \n');
fprintf(1,'Test of Spatial Dependence \n');
fprintf(1,' \n');
fprintf(1,' Test             Value      DF    Prob \n');
pj=1-chi2cdf(LMSURerr,T);
fprintf(1,'LM-SUR-err   = %9.4f',LMSURerr); fprintf(1,'    %2.0f',T); fprintf(1,' %9.4f \n',1-chi2cdf(LMSURerr,T));
fprintf(1,'LM*-SUR-err  = %9.4f',LMRSURerr);fprintf(1,'    %2.0f',T); fprintf(1,' %9.4f \n',1-chi2cdf(LMRSURerr,T));
fprintf(1,'LM-SUR-lag   = %9.4f',LMSURlag);fprintf(1,'    %2.0f',T); fprintf(1,' %9.4f \n',1-chi2cdf(LMSURlag,T));
fprintf(1,'LM*-SUR-lag  = %9.4f',LMRSURlag);fprintf(1,'    %2.0f',T); fprintf(1,' %9.4f \n',1-chi2cdf(LMRSURlag,T));
fprintf(1,'LM-SUR-Sarma = %9.4f',LMSURsarma);fprintf(1,'    %2.0f',2*T); fprintf(1,' %9.4f \n',1-chi2cdf(LMSURsarma,2*T));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Estimacion del Modelo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bt=[jj tmp(1:k*T+T) tstat pv];
results.bt=bt;
fprintf(1,' \n');
fprintf(1,'ML ESTIMATION SEEMINGLY UNRELATED REGRESSIONS (SUR)\n');
fprintf(1,'MODELS WITH SPATIALLY LAGGED DEPENDENT VARIABLE \n');
fprintf(1,' \n');
fprintf(1,'Lik without spatial effect %9.4f \n',LL);
fprintf(1,'Lik with spatial effect    %9.4f \n',-fvalh1);
fprintf(1,'                             Test LR       DF      Prob \n');
fprintf(1,'Likelihook Ratio           %9.4f',-2*(LL+fvalh1));fprintf(1,'       %2.0f',T);fprintf(1,' %9.4f \n',1-chi2cdf(-2*(LL+fvalh1),T));
fprintf(1,' \n');
for i=1:T
fprintf(1,'Ecuacin %2.0f',i);fprintf(1,'    Observaciones %2.0f',n);fprintf(1,'    DF %2.0f',n-k-1);fprintf(1,'    N var %2.0f \n',k);

fprintf(1,'Variable   Coeff     S.D.    t-value  p-value \n');
for j=1:k
fprintf(1,'Variable');fprintf(1,'%9.4f',bt(k*(i-1)+j,:));fprintf(1,'\n');
end
fprintf(1,'lambda  ');fprintf(1,'%9.4f',bt(k*T+i,:));fprintf(1,'\n');
fprintf(1,' \n');
end
% fprintf(1,'ERROR COVARIANCE MATRIX \n');
% SH1
fprintf(1,'ERROR CORRELATION MATRIX \n');
disp(CH1)
results.e=LMSURerr;
results.er=LMRSURerr;
results.l=LMSURlag;
results.lr=LMRSURlag;
results.s=LMSURsarma;
results.t=[LMSURerr LMRSURerr LMSURlag LMRSURlag LMSURsarma];

