function results = sur_spdiag(y,X,W,info)

% PURPOSE: computes seemingly unrelated regression estimates
%          for a model with neqs-equations y obtener test de diagnostico de
%          esfericidad y dependencia espacial.
%---------------------------------------------------
% where: 
%        T   = # of equccations
%        y   = an 'eq' structure containing dependent variables 
%              e.g. y(1).eq = y1; y(2).eq = y2; y(3).eq = y3;
%        x   = an 'eq' structure containing explanatory variables
%                 e.g. x(1).eq = [iota x1 x4]; 
%                      x(2).eq = [iota x1]; 
%                      x(3).eq = [iota x1 x2 x5];
%         info = an (optional) structure variable with input options
%         info.print = 'yes' / ['no'] print the result
%
%  SEE ALSO: sur_lag0;sur_err0;...
% ---------------------------------------------------
% REFERENCES: 
% Ref: Mur J, Lpez FA and M Herrera (2010) Testing for spatial effect in 
% Seemingly Unrelated Regressions. Spatial Economic Analysis 5(4) 399-440
% ---------------------------------------------------
% written by:
% Fernando A. Lpez Hernndez, 01/03/2008
% Facultad de C.C. de la Empresa
% Dpto. Metodos Cuantitativos e Informticos
% Universidad Politcnica de Cartagena
% E-mail: Fernando.Lopez@upct.es
if nargin<4
    info.print='no';
else
    info.print=lower(info.print);
end
[n k]=size(X(1).eq); % En principio, paneles equilibrados: mismo k para todo t
T=length(y);
info.itmax = 10;
info.crit=0.0001;
Rsur=sur(T,y,X);% toolbox de Lesage para obtener la estimacin max ver del modelo SUR.
% NOTA: la toolbox de LeSage tiene un error y nombra results(eq).yhat a los residuos y
% results(eq).resid a los valores previstos.

for i=1:T
    u(i).eq=olse(y(i).eq,X(i).eq);
    v(i).eq=ols(y(i).eq,X(i).eq);
    s(i).eq=u(i).eq'*u(i).eq/n;
    L(i)=-(n/2)*log(2*pi)-(n/2)*log(s(i).eq)-(1/(2*s(i).eq))*u(i).eq'*u(i).eq; % Obtencin de los valores de la FV para cada ecuacin
end
% Valores Lik para cada una de las ecuaciones temporales.
% L;
% Apilamos los vectores 
vv=sparse(v(1).eq.resid);
for i=2:T
    vv=sparse(horzcat(vv,v(i).eq.resid));
end
yy=sparse(vertcat(y(1:T).eq));
B=sparse(vertcat(Rsur(1:T).beta));
XX=sparse(blkdiag(X(1:T).eq));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test de diagonalidad de la matriz de varianzas y cova
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Test LR de cociente de verosimiltudes para ver si los residuos estn correlacionados
S=sparse(Rsur(1).sigma);
SI=sparse(inv(S));
% K = kron(SI,eye(n,n));
r1=sparse(yy-XX*B);
LL=-(T*n/2)*log(2*pi)-(n/2)*log(det(S))-(1/2)*r1'*kron(SI,eye(n,n))*r1;
LR=-2*(sum(L)-LL);

% Test LM de esfericidad
r=corrcoef(vv);
LME=0;
for i=1:T-1
    for j=i+1:T
        LME=LME+n*r(i,j)^2;
    end
end

% results.t=[LME LR];
% 
% return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test de dependencia espacial
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM-SUR-Err
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

iota=sparse(ones(T,1));
U=sparse(horzcat(Rsur(1:T).yhat));
% Con la idea de Anselin 1988b
gerr=iota'*(SI.*(U'*W*U));
WW=sparse(W'*W);
tWW=trace(WW);
tW2=trace(W^2);
TTT=sparse(tW2*eye(T,T));
TTS=sparse(tWW*ones(T,T));
LMSURerr=full(gerr*inv(TTT+TTS.*(SI.*S))*gerr');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM-SUR-Lag
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:T
    for j=1:T
        E(i,j).eq=zeros(T,T);
        E(i,j).eq(i,j)=1;
        E(i,j).eq(j,i)=1;
    end
end
for i=1:T
    for j=1:T
        E(i,j).eq=sparse(E(i,j).eq);
    end
end
% gradiente
YL=sparse(W*y(1).eq);
for i=2:T
    YL=sparse(horzcat(YL,W*y(i).eq));
end
glag=iota'*(SI.*(U'*YL));
% Matriz de Informacin
XXB=sparse(XX*B);
JI11=XX'*sparse(kron(SI,eye(n)))*XX;
JI12=XX'*sparse(kron(SI*E(1,1).eq,W))*XXB;
for i=2:T
    JI12=sparse(horzcat(JI12,XX'*sparse(kron(SI*E(i,i).eq,W))*XXB));
end
JI22=zeros(T,T);
for i=1:T
    for j=1:T
        JI22(i,j)=sparse(XXB'*sparse(kron(E(j,j).eq*SI*E(i,i).eq,WW))*XXB + SI(i,j)*S(i,j)*tWW);
    end
end
JI22=sparse(JI22+TTT);
% Indexar los elementos diferentes de Sigma
TT=T*(T+1)/2;
c1=0;c2=0;c3=0;
f=zeros(1,TT);
c=zeros(1,TT);
for k1=1:T
    c2=c2+1;
    for i=1:T-k1+1
        c1=c1+1;c3=c3+1;
        f(c1)=c2;
        c(c1)=c3;
    end
    c3=c2;
end
JI33=zeros(TT,TT);
SI2=sparse(SI*SI);
 for i=1:TT
    for j=1:TT   
       JI33(i,j)=(n/2)*trace(SI2*E(f(i),c(i)).eq*E(f(j),c(j)).eq);
    end
 end
LMSURlag=full(glag*inv(JI22-JI12'*inv(JI11)*JI12)*glag');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM-SUR-SARMA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SI22=sparse(JI22);
SI23=sparse(TTT+TTS.*(SI.*S));
SI33=sparse(SI.*S*tWW+eye(T,T)*tWW);
gsarma=sparse([glag gerr]);
LMSURsarma=full(gsarma*sparse(inv([JI22-JI12'*inv(JI11)*JI12  SI23;SI23' SI33]))*gsarma');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM*-SUR-Lag
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Iff=sparse([JI11 zeros(k*T,TT);zeros(TT,k*T) JI33]);
Ilf=sparse([JI12' zeros(T,TT)]);
Ilpf=SI22-Ilf*inv(Iff)*Ilf';
Irr=SI33;
Irl=SI23;
IIrr=sparse(inv(Irr));
LMRSURlag=full((glag'-Irl*IIrr*gerr')'*inv(Ilpf-Irl*IIrr*Irl)*(glag'-Irl*IIrr*gerr'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Test LM*-SUR-Err
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
LMRSURerr=full((gerr'-Irl*inv(Ilpf)*glag')'*inv(Irr-Irl*inv(Ilpf)*Irl)*(gerr'-Irl*inv(Ilpf)*glag'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     IMPRESION RESULTADOS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
results.t=[LME LR LMSURerr LMRSURerr LMSURlag LMRSURlag LMSURsarma];
if strcmp(info.print,'yes')
fprintf(1,' \n');
fprintf(1,'Test of diagonality of error covariance matrix \n');
fprintf(1,' \n');
fprintf(1,' Test                        Value          DF    Prob \n');
fprintf(1,'LM - Lagrange Multiplier = %9.4f   ',LME);fprintf(1,'    %2.0f',T*(T-1)/2); fprintf(1,' %9.4f \n',1-chi2cdf(LME,T*(T-1)/2));
fprintf(1,'LR - Likelihood Ratio    = %9.4f   ',LR);fprintf(1,'    %2.0f',0); fprintf(1,' %9.4f \n',1-chi2cdf(LR,T*(T-1)/2));
fprintf(1,' \n');
fprintf(1,'Test of Spatial Dependence \n');
fprintf(1,' \n');
fprintf(1,' Test             Value      DF    Prob \n');
fprintf(1,'LM-SUR-err   = %9.4f',LMSURerr); fprintf(1,'    %2.0f',T); fprintf(1,' %9.4f \n',1-chi2cdf(LMSURerr,T));
fprintf(1,'LM*-SUR-err  = %9.4f',LMRSURerr);fprintf(1,'    %2.0f',T); fprintf(1,' %9.4f \n',1-chi2cdf(LMRSURerr,T));
fprintf(1,'LM-SUR-lag   = %9.4f',LMSURlag);fprintf(1,'    %2.0f',T); fprintf(1,' %9.4f \n',1-chi2cdf(LMSURlag,T));
fprintf(1,'LM*-SUR-lag  = %9.4f',LMRSURlag);fprintf(1,'    %2.0f',T); fprintf(1,' %9.4f \n',1-chi2cdf(LMRSURlag,T));
fprintf(1,'LM-SUR-Sarma = %9.4f',LMSURsarma);fprintf(1,'    %2.0f',2*T); fprintf(1,' %9.4f \n',1-chi2cdf(LMSURsarma,2*T));
end
